@@ -1,5 +1,6 @@
 from enum import Enum
 import logging
+import functools
 import time
 import base64
 import json
@@ -22,6 +23,7 @@ from transform_tools.validate import (
 from .constants import DEFAULT_QUERY_TIMEOUT, MAX_PAGE_SIZE
 from .exceptions import QueryRuntimeException
 from .utils import resolve_local_commit_info
+from .helpers import intersect_dict
 from .models import (
     MqlQueryStatus,
     MqlQueryStatusResp,
@@ -739,39 +741,70 @@ class MQLInterface(BaseInterface):  # noqa: D
             )["metricByName"]
         )
 
-    def get_dimensions(self, model_key: Optional[ModelKey] = None) -> List[Dimension]:
+    def get_dimensions(
+        self,
+        model_key: Optional[ModelKey] = None,
+        metric_names: Optional[List[str]] = None,
+    ) -> List[Dimension]:
         """Get a list of all unique Dimension objects."""
-        query = gql(
-            """
-            query DimensionsQuery($modelKey: ModelKeyInput) {
-                dimensions(modelKey: $modelKey) {
-                    name
-                    identifierName
-                    identifierNames
-                    type
-                    isPrimaryTime
-                    timeGranularity
-                    cardinality
-                }
+        variable_values: Dict[str, Any] = {  # type: ignore[misc]
+            "modelKey": {
+                "organization": model_key.organization_id,
+                "repo": model_key.repository,
+                "branch": model_key.branch,
+                "commit": model_key.commit,
             }
-            """
-        )
-        return [
-            Dimension.from_gql_dict(dim)
-            for dim in self.execute(
-                query,
-                variable_values={
-                    "modelKey": {
-                        "organization": model_key.organization_id,
-                        "repo": model_key.repository,
-                        "branch": model_key.branch,
-                        "commit": model_key.commit,
+            if model_key
+            else None,
+        }
+        if metric_names:
+            dims = []
+            # TODO: add a gql query in MQS to get a list of metrics given a list of metric names
+            for metric_name in metric_names:
+                query = gql(
+                    """
+                        query MetricDimensionQuery($modelKey: ModelKeyInput, $metricName: String!) {
+                            metricByName(modelKey: $modelKey, name: $metricName) {
+                                dimensionObjects {
+                                    name
+                                    identifierName
+                                    identifierNames
+                                    type
+                                    isPrimaryTime
+                                    timeGranularity
+                                    cardinality
+                                }
+                            }
+                        }
+                    """
+                )
+                variable_values["metricName"] = metric_name
+                resp = self.execute(query, variable_values=variable_values)["metricByName"]["dimensionObjects"]
+                dims.append({dim["name"]: dim for dim in resp})
+                dims = [functools.reduce(lambda x, y: intersect_dict(x, y), dims)]
+                if len(dims) == 0:
+                    # No intersection, early return
+                    return []
+            dims = list(dims[0].values())
+        else:
+            query = gql(
+                """
+                    query DimensionsQuery($modelKey: ModelKeyInput) {
+                        dimensions(modelKey: $modelKey) {
+                            name
+                            identifierName
+                            identifierNames
+                            type
+                            isPrimaryTime
+                            timeGranularity
+                            cardinality
+                        }
                     }
-                    if model_key
-                    else None
-                },
-            )["dimensions"]
-        ]
+                """
+            )
+            dims = self.execute(query, variable_values=variable_values)["dimensions"]
+
+        return [Dimension.from_gql_dict(dim) for dim in dims]
 
     def get_data_warehouse_validation_issues(
         self, dw_validation_type: DWValidationType, model_key: Optional[ModelKey] = None
