@@ -287,10 +287,14 @@ class MQLClient:
             metric_name, dimension_name, model_key=model_key, page_number=page_number, page_size=page_size
         )
 
-    def list_dimensions(self, model_key_id: Optional[int] = None) -> Dict[str, Dimension]:
+    def list_dimensions(
+        self,
+        model_key_id: Optional[int] = None,
+        metric_names: Optional[List[str]] = None,
+    ) -> Dict[str, Dimension]:
         """Retrieves a list of all Dimension objects."""
         model_key = self.get_model_key(model_key_id) if model_key_id else None
-        dimensions = self.context.mql_client.get_dimensions(model_key)
+        dimensions = self.context.mql_client.get_dimensions(model_key, metric_names)
         return {dim.name: dim for dim in dimensions}
 
     def get_data_warehouse_issues(self, type: DWValidationType, model_key_id: Optional[int] = None) -> List[str]:
